@@ -12,14 +12,14 @@
 /* formatting helper functions */
 
 gchar *
-gfu_common_checksum_format (const gchar *checksum)
+gfu_common_checksum_format(const gchar *checksum)
 {
 	const gchar *checksum_type;
 	guint len;
 	/* guess type based on length */
 	if (checksum == NULL)
 		return NULL;
-	len = strlen (checksum);
+	len = strlen(checksum);
 	if (len == 32)
 		checksum_type = "MD5";
 	else if (len == 40)
@@ -30,13 +30,11 @@ gfu_common_checksum_format (const gchar *checksum)
 		checksum_type = "SHA512";
 	else
 		checksum_type = "SHA1";
-	return g_strdup_printf ("%s(%s)",
-				checksum_type,
-				checksum);
+	return g_strdup_printf("%s(%s)", checksum_type, checksum);
 }
 
 const gchar *
-gfu_common_seconds_to_string (guint64 seconds)
+gfu_common_seconds_to_string(guint64 seconds)
 {
 	guint64 minutes, hours;
 	if (seconds == 0)
@@ -47,295 +45,203 @@ gfu_common_seconds_to_string (guint64 seconds)
 		if (minutes >= 60) {
 			hours = minutes / 60;
 			minutes = minutes % 60;
-			return g_strdup_printf ("%lu hr, %lu min, %lu sec", hours, minutes, seconds);
+			return g_strdup_printf("%lu hr, %lu min, %lu sec", hours, minutes, seconds);
 		}
-		return g_strdup_printf ("%lu min, %lu sec", minutes, seconds);
+		return g_strdup_printf("%lu min, %lu sec", minutes, seconds);
 	}
-	return g_strdup_printf ("%lu sec", seconds);
+	return g_strdup_printf("%lu sec", seconds);
 }
 
 gchar *
-gfu_common_xml_to_text (const gchar *xml, GError **error)
+gfu_common_xml_to_text(const gchar *xml, GError **error)
 {
-	g_autoptr(GString) str = g_string_new (NULL);
+	g_autoptr(GString) str = g_string_new(NULL);
 	g_autoptr(XbNode) n = NULL;
 	g_autoptr(XbSilo) silo = NULL;
 
 	if (xml == NULL) {
-		g_set_error_literal (error,
-				     FWUPD_ERROR,
-				     FWUPD_ERROR_INTERNAL,
-				     "Null input XML");
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_INTERNAL, "Null input XML");
 		return NULL;
 	}
 
 	/* parse XML */
-	silo = xb_silo_new_from_xml (xml, error);
+	silo = xb_silo_new_from_xml(xml, error);
 	if (silo == NULL)
 		return NULL;
 
-	n = xb_silo_get_root (silo);
+	n = xb_silo_get_root(silo);
 	while (n != NULL) {
 		g_autoptr(XbNode) n2 = NULL;
 
 		/* support <p>, <ul>, <ol> and <li>, ignore all else */
-		if (g_strcmp0 (xb_node_get_element (n), "p") == 0) {
-			g_string_append_printf (str, "%s\n\n", xb_node_get_text (n));
-		} else if (g_strcmp0 (xb_node_get_element (n), "ul") == 0) {
-			g_autoptr(GPtrArray) children = xb_node_get_children (n);
+		if (g_strcmp0(xb_node_get_element(n), "p") == 0) {
+			g_string_append_printf(str, "%s\n\n", xb_node_get_text(n));
+		} else if (g_strcmp0(xb_node_get_element(n), "ul") == 0) {
+			g_autoptr(GPtrArray) children = xb_node_get_children(n);
 			for (guint i = 0; i < children->len; i++) {
-				XbNode *nc = g_ptr_array_index (children, i);
-				if (g_strcmp0 (xb_node_get_element (nc), "li") == 0) {
-					g_string_append_printf (str, " • %s\n",
-								xb_node_get_text (nc));
+				XbNode *nc = g_ptr_array_index(children, i);
+				if (g_strcmp0(xb_node_get_element(nc), "li") == 0) {
+					g_string_append_printf(str,
+							       " • %s\n",
+							       xb_node_get_text(nc));
 				}
 			}
-			g_string_append (str, "\n");
-		} else if (g_strcmp0 (xb_node_get_element (n), "ol") == 0) {
-			g_autoptr(GPtrArray) children = xb_node_get_children (n);
+			g_string_append(str, "\n");
+		} else if (g_strcmp0(xb_node_get_element(n), "ol") == 0) {
+			g_autoptr(GPtrArray) children = xb_node_get_children(n);
 			for (guint i = 0; i < children->len; i++) {
-				XbNode *nc = g_ptr_array_index (children, i);
-				if (g_strcmp0 (xb_node_get_element (nc), "li") == 0) {
-					g_string_append_printf (str, " %u. %s\n",
-								i + 1,
-								xb_node_get_text (nc));
+				XbNode *nc = g_ptr_array_index(children, i);
+				if (g_strcmp0(xb_node_get_element(nc), "li") == 0) {
+					g_string_append_printf(str,
+							       " %u. %s\n",
+							       i + 1,
+							       xb_node_get_text(nc));
 				}
 			}
-			g_string_append (str, "\n");
+			g_string_append(str, "\n");
 		}
 
-		n2 = xb_node_get_next (n);
-		g_set_object (&n, n2);
+		n2 = xb_node_get_next(n);
+		g_set_object(&n, n2);
 	}
 
 	/* remove extra newline */
 	if (str->len > 0)
-		g_string_truncate (str, str->len - 1);
+		g_string_truncate(str, str->len - 1);
 
 	/* success */
-	return g_string_free (g_steal_pointer (&str), FALSE);
+	return g_string_free(g_steal_pointer(&str), FALSE);
 }
 
 const gchar *
-gfu_common_device_flags_to_strings (guint64 flags)
+gfu_common_device_flags_to_strings(guint64 flags)
 {
-	g_autoptr(GString) str = g_string_new (NULL);
+	g_autoptr(GString) str = g_string_new(NULL);
 	for (guint j = 0; j < 64; j++) {
-		if ((flags & ((guint64) 1 << j)) == 0)
+		if ((flags & ((guint64)1 << j)) == 0)
 			continue;
-		g_string_append_printf (str, "%s\n",
-					fwupd_device_flag_to_string ((guint64) 1 << j));
+		g_string_append_printf(str, "%s\n", fwupd_device_flag_to_string((guint64)1 << j));
 	}
 	if (str->len == 0) {
-		g_string_append (str, fwupd_device_flag_to_string (0));
+		g_string_append(str, fwupd_device_flag_to_string(0));
 	} else {
-		g_string_truncate (str, str->len - 1);
+		g_string_truncate(str, str->len - 1);
 	}
-	return g_string_free (g_steal_pointer (&str), FALSE);
+	return g_string_free(g_steal_pointer(&str), FALSE);
 }
 
 const gchar *
-gfu_common_release_flags_to_strings (guint64 flags)
+gfu_common_release_flags_to_strings(guint64 flags)
 {
-	g_autoptr(GString) str = g_string_new (NULL);
+	g_autoptr(GString) str = g_string_new(NULL);
 	for (guint j = 0; j < 64; j++) {
-		if ((flags & ((guint64) 1 << j)) == 0)
+		if ((flags & ((guint64)1 << j)) == 0)
 			continue;
-		g_string_append_printf (str, "%s\n",
-					fwupd_device_flag_to_string ((guint64) 1 << j));
+		g_string_append_printf(str, "%s\n", fwupd_device_flag_to_string((guint64)1 << j));
 	}
 	if (str->len == 0) {
-		g_string_append (str, fwupd_release_flag_to_string (0));
+		g_string_append(str, fwupd_release_flag_to_string(0));
 	} else {
-		g_string_truncate (str, str->len - 1);
+		g_string_truncate(str, str->len - 1);
 	}
-	return g_string_free (g_steal_pointer (&str), FALSE);
+	return g_string_free(g_steal_pointer(&str), FALSE);
 }
 
 /* handle needs-reboot and needs-shutdown */
 
 gboolean
-gfu_common_system_shutdown (GError **error)
+gfu_common_system_shutdown(GError **error)
 {
 	g_autoptr(GDBusConnection) connection = NULL;
 	g_autoptr(GVariant) val = NULL;
 
-	connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, error);
+	connection = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, error);
 	if (connection == NULL)
 		return FALSE;
 
 #ifdef HAVE_LOGIND
 	/* shutdown using logind */
-	val = g_dbus_connection_call_sync (connection,
-					   "org.freedesktop.login1",
-					   "/org/freedesktop/login1",
-					   "org.freedesktop.login1.Manager",
-					   "PowerOff",
-					   g_variant_new ("(b)", TRUE),
-					   NULL,
-					   G_DBUS_CALL_FLAGS_NONE,
-					   -1,
-					   NULL,
-					   error);
+	val = g_dbus_connection_call_sync(connection,
+					  "org.freedesktop.login1",
+					  "/org/freedesktop/login1",
+					  "org.freedesktop.login1.Manager",
+					  "PowerOff",
+					  g_variant_new("(b)", TRUE),
+					  NULL,
+					  G_DBUS_CALL_FLAGS_NONE,
+					  -1,
+					  NULL,
+					  error);
 #elif defined(HAVE_CONSOLEKIT)
 	/* shutdown using ConsoleKit */
-	val = g_dbus_connection_call_sync (connection,
-					   "org.freedesktop.ConsoleKit",
-					   "/org/freedesktop/ConsoleKit/Manager",
-					   "org.freedesktop.ConsoleKit.Manager",
-					   "Stop",
-					   NULL,
-					   NULL,
-					   G_DBUS_CALL_FLAGS_NONE,
-					   -1,
-					   NULL,
-					   error);
+	val = g_dbus_connection_call_sync(connection,
+					  "org.freedesktop.ConsoleKit",
+					  "/org/freedesktop/ConsoleKit/Manager",
+					  "org.freedesktop.ConsoleKit.Manager",
+					  "Stop",
+					  NULL,
+					  NULL,
+					  G_DBUS_CALL_FLAGS_NONE,
+					  -1,
+					  NULL,
+					  error);
 #else
-	g_set_error_literal (error,
-			     FWUPD_ERROR,
-			     FWUPD_ERROR_INVALID_ARGS,
-			     "No supported backend compiled in to perform the operation.");
+	g_set_error_literal(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INVALID_ARGS,
+			    "No supported backend compiled in to perform the operation.");
 #endif
 	return val != NULL;
 }
 
 gboolean
-gfu_common_system_reboot (GError **error)
+gfu_common_system_reboot(GError **error)
 {
 	g_autoptr(GDBusConnection) connection = NULL;
 	g_autoptr(GVariant) val = NULL;
 
-	connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, error);
+	connection = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, error);
 	if (connection == NULL)
 		return FALSE;
 #ifdef HAVE_LOGIND
 	/* reboot using logind */
-	val = g_dbus_connection_call_sync (connection,
-					   "org.freedesktop.login1",
-					   "/org/freedesktop/login1",
-					   "org.freedesktop.login1.Manager",
-					   "Reboot",
-					   g_variant_new ("(b)", TRUE),
-					   NULL,
-					   G_DBUS_CALL_FLAGS_NONE,
-					   -1,
-					   NULL,
-					   error);
+	val = g_dbus_connection_call_sync(connection,
+					  "org.freedesktop.login1",
+					  "/org/freedesktop/login1",
+					  "org.freedesktop.login1.Manager",
+					  "Reboot",
+					  g_variant_new("(b)", TRUE),
+					  NULL,
+					  G_DBUS_CALL_FLAGS_NONE,
+					  -1,
+					  NULL,
+					  error);
 #elif defined(HAVE_CONSOLEKIT)
 	/* reboot using ConsoleKit */
-	val = g_dbus_connection_call_sync (connection,
-					   "org.freedesktop.ConsoleKit",
-					   "/org/freedesktop/ConsoleKit/Manager",
-					   "org.freedesktop.ConsoleKit.Manager",
-					   "Restart",
-					   NULL,
-					   NULL,
-					   G_DBUS_CALL_FLAGS_NONE,
-					   -1,
-					   NULL,
-					   error);
+	val = g_dbus_connection_call_sync(connection,
+					  "org.freedesktop.ConsoleKit",
+					  "/org/freedesktop/ConsoleKit/Manager",
+					  "org.freedesktop.ConsoleKit.Manager",
+					  "Restart",
+					  NULL,
+					  NULL,
+					  G_DBUS_CALL_FLAGS_NONE,
+					  -1,
+					  NULL,
+					  error);
 #else
-	g_set_error_literal (error,
-			     FWUPD_ERROR,
-			     FWUPD_ERROR,
-			     "No supported backend compiled in to perform the operation.");
+	g_set_error_literal(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR,
+			    "No supported backend compiled in to perform the operation.");
 #endif
 	return val != NULL;
 }
 
-/* installation helper functions */
-
-gboolean
-gfu_common_mkdir_parent (const gchar *filename, GError **error)
-{
-	g_autofree gchar *parent = NULL;
-
-	parent = g_path_get_dirname (filename);
-	g_debug ("creating path %s", parent);
-	if (g_mkdir_with_parents (parent, 0755) == -1) {
-		g_set_error (error,
-			     FWUPD_ERROR,
-			     FWUPD_ERROR_INTERNAL,
-			     "Failed to create '%s': %s",
-			     parent, g_strerror (errno));
-		return FALSE;
-	}
-	return TRUE;
-}
-
-gchar *
-gfu_get_user_cache_path (const gchar *fn)
-{
-	g_autofree gchar *basename = g_path_get_basename (fn);
-
-	return g_build_filename (g_get_user_cache_dir (), "gfu", basename, NULL);
-}
-
-gboolean
-gfu_common_file_exists_with_checksum (const gchar *fn,
-				   const gchar *checksum_expected,
-				   GChecksumType checksum_type)
-{
-	gsize len = 0;
-	g_autofree gchar *checksum_actual = NULL;
-	g_autofree gchar *data = NULL;
-
-	if (!g_file_get_contents (fn, &data, &len, NULL))
-		return FALSE;
-	checksum_actual = g_compute_checksum_for_data (checksum_type,
-						       (guchar *) data, len);
-	return g_strcmp0 (checksum_expected, checksum_actual) == 0;
-}
-
-SoupSession *
-gfu_common_setup_networking (GError **error)
-{
-	const gchar *http_proxy;
-	g_autofree gchar *user_agent = NULL;
-	g_autoptr(SoupSession) session = NULL;
-
-	/* create the soup session */
-	user_agent = fwupd_build_user_agent (GETTEXT_PACKAGE, VERSION);
-	session = soup_session_new_with_options (SOUP_SESSION_USER_AGENT, user_agent,
-						 SOUP_SESSION_TIMEOUT, 60,
-						 NULL);
-	if (session == NULL) {
-		g_set_error_literal (error,
-				     FWUPD_ERROR,
-				     FWUPD_ERROR_INTERNAL,
-				     "failed to setup networking");
-		return NULL;
-	}
-
-	/* set the proxy */
-	http_proxy = g_getenv ("https_proxy");
-	if (http_proxy == NULL)
-		http_proxy = g_getenv ("HTTPS_PROXY");
-	if (http_proxy == NULL)
-		http_proxy = g_getenv ("http_proxy");
-	if (http_proxy == NULL)
-		http_proxy = g_getenv ("HTTP_PROXY");
-	if (http_proxy != NULL && strlen (http_proxy) > 0) {
-		g_autoptr(SoupURI) proxy_uri = soup_uri_new (http_proxy);
-		if (proxy_uri == NULL) {
-			g_set_error (error,
-				     FWUPD_ERROR,
-				     FWUPD_ERROR_INTERNAL,
-				     "invalid proxy URI: %s", http_proxy);
-			return NULL;
-		}
-		g_object_set (session, SOUP_SESSION_PROXY_URI, proxy_uri, NULL);
-	}
-
-	/* this disables the double-compression of the firmware.xml.gz file */
-	soup_session_remove_feature_by_type (session, SOUP_TYPE_CONTENT_DECODER);
-	return g_steal_pointer (&session);
-}
-
 /* GTK helper functions */
 
 gchar *
-gfu_common_device_flag_to_string (guint64 device_flag)
+gfu_common_device_flag_to_string(guint64 device_flag)
 {
 	if (device_flag == FWUPD_DEVICE_FLAG_NONE) {
 		return NULL;
@@ -354,7 +260,7 @@ gfu_common_device_flag_to_string (guint64 device_flag)
 	}
 	if (device_flag == FWUPD_DEVICE_FLAG_REQUIRE_AC) {
 		/* TRANSLATORS: Must be plugged in to an outlet */
-		return _("Requires AC power");
+		return _("System requires external power source");
 	}
 	if (device_flag == FWUPD_DEVICE_FLAG_LOCKED) {
 		/* TRANSLATORS: Is locked and can be unlocked */
@@ -420,13 +326,10 @@ gfu_common_device_flag_to_string (guint64 device_flag)
 		/* skip */
 		return NULL;
 	}
-#if FWUPD_CHECK_VERSION(1,3,2)
 	if (device_flag == FWUPD_DEVICE_FLAG_HISTORICAL) {
 		/* skip */
 		return NULL;
 	}
-#endif
-#if FWUPD_CHECK_VERSION(1,3,3)
 	if (device_flag == FWUPD_DEVICE_FLAG_ONLY_SUPPORTED) {
 		/* skip */
 		return NULL;
@@ -455,6 +358,49 @@ gfu_common_device_flag_to_string (guint64 device_flag)
 		/* TRANSLATORS: Device remains usable during update */
 		return _("Device is usable for the duration of the update");
 	}
+	if (device_flag == FWUPD_DEVICE_FLAG_INSTALL_ALL_RELEASES) {
+		/* TRANSLATORS: Device does not jump directly to the newest */
+		return _("Device installs all intermediate releases");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_SKIPS_RESTART) {
+		/* TRANSLATORS: Device does not restart and requires a manual action */
+		return _("Device skips restart");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_HAS_MULTIPLE_BRANCHES) {
+		/* TRANSLATORS: the stream can be provided by different vendors */
+		return _("Device supports switching to a different stream of firmware");
+	}
+	if (device_flag == FWUPD_DEVICE_FLAG_BACKUP_BEFORE_INSTALL) {
+		/* TRANSLATORS: it is saved to the users local disk */
+		return _("Device firmware will be saved before installing updates");
+	}
+#if FWUPD_CHECK_VERSION(1, 6, 2)
+	if (device_flag == FWUPD_DEVICE_FLAG_WILDCARD_INSTALL) {
+		/* TRANSLATORS: on some systems certain devices have to have matching versions,
+		 * e.g. the EFI driver for a given network card cannot be different */
+		return _("All devices of the same type will be updated at the same time");
+	}
+#endif
+#if FWUPD_CHECK_VERSION(1, 6, 2)
+	if (device_flag == FWUPD_DEVICE_FLAG_ONLY_VERSION_UPGRADE) {
+		/* TRANSLATORS: some devices can only be updated to a new semver and cannot
+		 * be downgraded or reinstalled with the sexisting version */
+		return _("Only version upgrades are allowed");
+	}
+#endif
+#if FWUPD_CHECK_VERSION(1, 7, 0)
+	if (device_flag == FWUPD_DEVICE_FLAG_UNREACHABLE) {
+		/* TRANSLATORS: the device is currently unreachable, perhaps because it is in a
+		 * lower power state or is out of wireless range */
+		return _("Device is currently unreachable");
+	}
+#endif
+#if FWUPD_CHECK_VERSION(1, 7, 1)
+	if (device_flag == FWUPD_DEVICE_FLAG_AFFECTS_FDE) {
+		/* TRANSLATORS: a volume with full-disk-encryption was found on this machine,
+		 * typically a Windows NTFS partition with BitLocker */
+		return _("May invalidate secrets used to decrypt volumes");
+	}
 #endif
 	if (device_flag == FWUPD_DEVICE_FLAG_UNKNOWN) {
 		return NULL;
@@ -463,7 +409,7 @@ gfu_common_device_flag_to_string (guint64 device_flag)
 }
 
 gchar *
-gfu_common_device_icon_from_flag (FwupdDeviceFlags device_flag)
+gfu_common_device_icon_from_flag(FwupdDeviceFlags device_flag)
 {
 	if (device_flag == FWUPD_DEVICE_FLAG_INTERNAL)
 		return "drive-harddisk-symbolic";
@@ -505,7 +451,6 @@ gfu_common_device_icon_from_flag (FwupdDeviceFlags device_flag)
 		return "emblem-important-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_ENSURE_SEMVER)
 		return "emblem-important-symbolic";
-#if FWUPD_CHECK_VERSION(1,3,3)
 	if (device_flag == FWUPD_DEVICE_FLAG_WILL_DISAPPEAR)
 		return "emblem-important-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_CAN_VERIFY)
@@ -516,7 +461,29 @@ gfu_common_device_icon_from_flag (FwupdDeviceFlags device_flag)
 		return "emblem-important-symbolic";
 	if (device_flag == FWUPD_DEVICE_FLAG_USABLE_DURING_UPDATE)
 		return "emblem-important-symbolic";
-
+	if (device_flag == FWUPD_DEVICE_FLAG_INSTALL_ALL_RELEASES)
+		return "media-skip-forward-symbolic";
+	if (device_flag == FWUPD_DEVICE_FLAG_SKIPS_RESTART)
+		return "process-stop-symbolic";
+	if (device_flag == FWUPD_DEVICE_FLAG_HAS_MULTIPLE_BRANCHES)
+		return "emblem-shared-symbolic";
+	if (device_flag == FWUPD_DEVICE_FLAG_BACKUP_BEFORE_INSTALL)
+		return "drive-harddisk-symbolic";
+#if FWUPD_CHECK_VERSION(1, 6, 2)
+	if (device_flag == FWUPD_DEVICE_FLAG_WILDCARD_INSTALL)
+		return "edit-copy-symbolic";
+#endif
+#if FWUPD_CHECK_VERSION(1, 6, 2)
+	if (device_flag == FWUPD_DEVICE_FLAG_ONLY_VERSION_UPGRADE)
+		return "go-up-symbolic";
+#endif
+#if FWUPD_CHECK_VERSION(1, 7, 0)
+	if (device_flag == FWUPD_DEVICE_FLAG_UNREACHABLE)
+		return "network-offline-symbolic";
+#endif
+#if FWUPD_CHECK_VERSION(1, 7, 1)
+	if (device_flag == FWUPD_DEVICE_FLAG_AFFECTS_FDE)
+		return "system-lock-screen-symbolic";
 #endif
 	if (device_flag == FWUPD_DEVICE_FLAG_UNKNOWN)
 		return "unknown-symbolic";
@@ -524,28 +491,25 @@ gfu_common_device_icon_from_flag (FwupdDeviceFlags device_flag)
 }
 
 gchar *
-gfu_operation_to_string (GfuOperation operation, FwupdDevice *dev)
+gfu_operation_to_string(GfuOperation operation, FwupdDevice *dev)
 {
 	/* show message in progressbar */
 	if (operation == GFU_OPERATION_UPDATE) {
 		/* TRANSLATORS: %1 is a device name */
-		return g_strdup_printf (_("Updating %s…"),
-					fwupd_device_get_name (dev));
+		return g_strdup_printf(_("Updating %s…"), fwupd_device_get_name(dev));
 	} else if (operation == GFU_OPERATION_DOWNGRADE) {
 		/* TRANSLATORS: %1 is a device name */
-		return g_strdup_printf (_("Downgrading %s…"),
-					fwupd_device_get_name (dev));
+		return g_strdup_printf(_("Downgrading %s…"), fwupd_device_get_name(dev));
 	} else if (operation == GFU_OPERATION_INSTALL) {
 		/* TRANSLATORS: %1 is a device name  */
-		return g_strdup_printf (_("Installing on %s…"),
-					fwupd_device_get_name (dev));
+		return g_strdup_printf(_("Installing on %s…"), fwupd_device_get_name(dev));
 	}
-	g_warning ("no GfuOperation set");
+	g_warning("no GfuOperation set");
 	return NULL;
 }
 
 const gchar *
-gfu_status_to_string (FwupdStatus status)
+gfu_status_to_string(FwupdStatus status)
 {
 	switch (status) {
 	case FWUPD_STATUS_IDLE:
@@ -596,6 +560,10 @@ gfu_status_to_string (FwupdStatus status)
 		/* TRANSLATORS: waiting for device to do something */
 		return _("Waiting…");
 		break;
+	case FWUPD_STATUS_SHUTDOWN:
+		/* TRANSLATORS: waiting for daemon */
+		return _("Shutting down…");
+		break;
 	default:
 		break;
 	}
@@ -603,3 +571,16 @@ gfu_status_to_string (FwupdStatus status)
 	/* TRANSLATORS: currect daemon status is unknown */
 	return _("Unknown");
 }
+
+gchar *
+gfu_common_strjoin_array(const gchar *separator, GPtrArray *array)
+{
+	g_autofree const gchar **strv = NULL;
+
+	g_return_val_if_fail(array != NULL, NULL);
+
+	strv = g_new0(const gchar *, array->len + 1);
+	for (guint i = 0; i < array->len; i++)
+		strv[i] = g_ptr_array_index(array, i);
+	return g_strjoinv(separator, (gchar **)strv);
+}
